@@ -67,17 +67,6 @@ static const char *guid_str(uint64_t node_guid, char *str)
 	return str;
 }
 
-static const char *port_state_str(enum ibv_port_state pstate)
-{
-	switch (pstate) {
-	case IBV_PORT_DOWN:   return "PORT_DOWN";
-	case IBV_PORT_INIT:   return "PORT_INIT";
-	case IBV_PORT_ARMED:  return "PORT_ARMED";
-	case IBV_PORT_ACTIVE: return "PORT_ACTIVE";
-	default:              return "invalid state";
-	}
-}
-
 static const char *port_phy_state_str(uint8_t phys_state)
 {
 	switch (phys_state) {
@@ -135,6 +124,18 @@ static const char *speed_str(uint8_t speed)
 	}
 }
 
+static const char *vl_str(uint8_t vl_num)
+{
+	switch (vl_num) {
+	case 1:  return "1";
+	case 2:  return "2";
+	case 3:  return "4";
+	case 4:  return "8";
+	case 5:  return "15";
+	default: return "invalid value";
+	}
+}
+
 static int print_all_port_gids(struct ibv_context *ctx, uint8_t port_num, int tbl_len)
 {
 	union ibv_gid gid;
@@ -254,7 +255,7 @@ static int print_hca_cap(struct ibv_device *ib_dev, uint8_t ib_port)
 		}
 		printf("\t\tport:\t%d\n", port);
 		printf("\t\t\tstate:\t\t\t%s (%d)\n",
-		       port_state_str(port_attr.state), port_attr.state);
+		       ibv_port_state_str(port_attr.state), port_attr.state);
 		printf("\t\t\tmax_mtu:\t\t%s (%d)\n",
 		       mtu_str(port_attr.max_mtu), port_attr.max_mtu);
 		printf("\t\t\tactive_mtu:\t\t%s (%d)\n",
@@ -266,7 +267,8 @@ static int print_hca_cap(struct ibv_device *ib_dev, uint8_t ib_port)
 		if (verbose) {
 			printf("\t\t\tmax_msg_sz:\t\t0x%x\n", port_attr.max_msg_sz);
 			printf("\t\t\tport_cap_flags:\t\t0x%08x\n", port_attr.port_cap_flags);
-			printf("\t\t\tmax_vl_num:\t\t%d\n", port_attr.max_vl_num);
+			printf("\t\t\tmax_vl_num:\t\t%s (%d)\n",
+			       vl_str(port_attr.max_vl_num), port_attr.max_vl_num);
 			printf("\t\t\tbad_pkey_cntr:\t\t0x%x\n", port_attr.bad_pkey_cntr);
 			printf("\t\t\tqkey_viol_cntr:\t\t0x%x\n", port_attr.qkey_viol_cntr);
 			printf("\t\t\tsm_sl:\t\t\t%d\n", port_attr.sm_sl);
@@ -310,13 +312,13 @@ int main(int argc, char *argv[])
 {
 	char *ib_devname = NULL;
 	int ret = 0;
-	struct ibv_device **dev_list;
+	struct ibv_device **dev_list, **orig_dev_list;
 	int num_of_hcas;
 	int ib_port = 0;
 
 	/* parse command line options */
 	while (1) {
-	        int c;
+		int c;
 		static struct option long_options[] = {
 			{ .name = "ib-dev",   .has_arg = 1, .val = 'd' },
 			{ .name = "ib-port",  .has_arg = 1, .val = 'i' },
@@ -347,7 +349,7 @@ int main(int argc, char *argv[])
 			break;
 
 		case 'l':
-			dev_list = ibv_get_device_list(&num_of_hcas);
+			dev_list = orig_dev_list = ibv_get_device_list(&num_of_hcas);
 			if (!dev_list) {
 				fprintf(stderr, "Failed to get IB devices list");
 				return -1;
@@ -362,6 +364,9 @@ int main(int argc, char *argv[])
 			}
 
 			printf("\n");
+
+			ibv_free_device_list(orig_dev_list);
+
 			return 0;
 
 		default:
@@ -370,7 +375,7 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	dev_list = ibv_get_device_list(NULL);
+	dev_list = orig_dev_list = ibv_get_device_list(NULL);
 	if (!dev_list) {
 		fprintf(stderr, "Failed to get IB device list\n");
 		return -1;
@@ -404,5 +409,7 @@ int main(int argc, char *argv[])
 	if (ib_devname)
 		free(ib_devname);
 
+	ibv_free_device_list(orig_dev_list);
+
 	return ret;
 }
