@@ -76,7 +76,8 @@ struct pingpong_dest {
 };
 
 static int pp_connect_ctx(struct pingpong_context *ctx, int port, int my_psn,
-			  enum ibv_mtu mtu, struct pingpong_dest *dest)
+			  enum ibv_mtu mtu, int sl,
+			  struct pingpong_dest *dest)
 {
 	struct ibv_qp_attr attr = {
 		.qp_state		= IBV_QPS_RTR,
@@ -88,7 +89,7 @@ static int pp_connect_ctx(struct pingpong_context *ctx, int port, int my_psn,
 		.ah_attr		= {
 			.is_global	= 0,
 			.dlid		= dest->lid,
-			.sl		= 0,
+			.sl		= sl,
 			.src_path_bits	= 0,
 			.port_num	= port
 		}
@@ -146,6 +147,7 @@ static struct pingpong_dest *pp_client_exch_dest(const char *servername, int por
 
 	if (n < 0) {
 		fprintf(stderr, "%s for %s:%d\n", gai_strerror(n), servername, port);
+		free(service);
 		return NULL;
 	}
 
@@ -160,6 +162,7 @@ static struct pingpong_dest *pp_client_exch_dest(const char *servername, int por
 	}
 
 	freeaddrinfo(res);
+	free(service);
 
 	if (sockfd < 0) {
 		fprintf(stderr, "Couldn't connect to %s:%d\n", servername, port);
@@ -192,7 +195,8 @@ out:
 }
 
 static struct pingpong_dest *pp_server_exch_dest(struct pingpong_context *ctx,
-						 int ib_port, enum ibv_mtu mtu, int port,
+						 int ib_port, enum ibv_mtu mtu,
+						 int port, int sl,
 						 const struct pingpong_dest *my_dest)
 {
 	struct addrinfo *res, *t;
@@ -214,6 +218,7 @@ static struct pingpong_dest *pp_server_exch_dest(struct pingpong_context *ctx,
 
 	if (n < 0) {
 		fprintf(stderr, "%s for port %d\n", gai_strerror(n), port);
+		free(service);
 		return NULL;
 	}
 
@@ -232,6 +237,7 @@ static struct pingpong_dest *pp_server_exch_dest(struct pingpong_context *ctx,
 	}
 
 	freeaddrinfo(res);
+	free(service);
 
 	if (sockfd < 0) {
 		fprintf(stderr, "Couldn't listen to port %d\n", port);
@@ -259,7 +265,7 @@ static struct pingpong_dest *pp_server_exch_dest(struct pingpong_context *ctx,
 
 	sscanf(msg, "%x:%x:%x", &rem_dest->lid, &rem_dest->qpn, &rem_dest->psn);
 
-	if (pp_connect_ctx(ctx, ib_port, my_dest->psn, mtu, rem_dest)) {
+	if (pp_connect_ctx(ctx, ib_port, my_dest->psn, mtu, sl, rem_dest)) {
 		fprintf(stderr, "Couldn't connect to remote QP\n");
 		free(rem_dest);
 		rem_dest = NULL;
@@ -358,12 +364,12 @@ static struct pingpong_context *pp_init_ctx(struct ibv_device *ib_dev, int size,
 	}
 
 	{
-		struct ibv_qp_attr attr;
-
-		attr.qp_state        = IBV_QPS_INIT;
-		attr.pkey_index      = 0;
-		attr.port_num        = port;
-		attr.qp_access_flags = 0;
+		struct ibv_qp_attr attr = {
+			.qp_state        = IBV_QPS_INIT,
+			.pkey_index      = 0,
+			.port_num        = port,
+			.qp_access_flags = 0
+		};
 
 		if (ibv_modify_qp(ctx->qp, &attr,
 				  IBV_QP_STATE              |
@@ -473,6 +479,7 @@ static void usage(const char *argv0)
 	printf("  -m, --mtu=<size>       path MTU (default 1024)\n");
 	printf("  -r, --rx-depth=<dep>   number of receives to post at a time (default 500)\n");
 	printf("  -n, --iters=<iters>    number of exchanges (default 1000)\n");
+	printf("  -l, --sl=<sl>          service level value\n");
 	printf("  -e, --events           sleep on CQ events (default poll)\n");
 }
 
@@ -496,6 +503,7 @@ int main(int argc, char *argv[])
 	int                      routs;
 	int                      rcnt, scnt;
 	int                      num_cq_events = 0;
+	int                      sl = 0;
 
 	srand48(getpid() * time(NULL));
 
@@ -510,11 +518,12 @@ int main(int argc, char *argv[])
 			{ .name = "mtu",      .has_arg = 1, .val = 'm' },
 			{ .name = "rx-depth", .has_arg = 1, .val = 'r' },
 			{ .name = "iters",    .has_arg = 1, .val = 'n' },
+			{ .name = "sl",       .has_arg = 1, .val = 'l' },
 			{ .name = "events",   .has_arg = 0, .val = 'e' },
 			{ 0 }
 		};
 
-		c = getopt_long(argc, argv, "p:d:i:s:m:r:n:e", long_options, NULL);
+		c = getopt_long(argc, argv, "p:d:i:s:m:r:n:l:e", long_options, NULL);
 		if (c == -1)
 			break;
 
@@ -558,6 +567,10 @@ int main(int argc, char *argv[])
 			iters = strtol(optarg, NULL, 0);
 			break;
 
+		case 'l':
+			sl = strtol(optarg, NULL, 0);
+			break;
+
 		case 'e':
 			++use_event;
 			break;
@@ -631,7 +644,7 @@ int main(int argc, char *argv[])
 	if (servername)
 		rem_dest = pp_client_exch_dest(servername, port, &my_dest);
 	else
-		rem_dest = pp_server_exch_dest(ctx, ib_port, mtu, port, &my_dest);
+		rem_dest = pp_server_exch_dest(ctx, ib_port, mtu, port, sl, &my_dest);
 
 	if (!rem_dest)
 		return 1;
@@ -640,7 +653,7 @@ int main(int argc, char *argv[])
 	       rem_dest->lid, rem_dest->qpn, rem_dest->psn);
 
 	if (servername)
-		if (pp_connect_ctx(ctx, ib_port, my_dest.psn, mtu, rem_dest))
+		if (pp_connect_ctx(ctx, ib_port, my_dest.psn, mtu, sl, rem_dest))
 			return 1;
 
 	ctx->pending = PINGPONG_RECV_WRID;
@@ -696,7 +709,8 @@ int main(int argc, char *argv[])
 
 			for (i = 0; i < ne; ++i) {
 				if (wc[i].status != IBV_WC_SUCCESS) {
-					fprintf(stderr, "Failed status %d for wr_id %d\n",
+					fprintf(stderr, "Failed status %s (%d) for wr_id %d\n",
+						ibv_wc_status_str(wc[i].status),
 						wc[i].status, (int) wc[i].wr_id);
 					return 1;
 				}
