@@ -80,7 +80,7 @@ struct pingpong_dest {
 };
 
 static int pp_connect_ctx(struct pingpong_context *ctx, int port, enum ibv_mtu mtu,
-			  const struct pingpong_dest *my_dest,
+			  int sl, const struct pingpong_dest *my_dest,
 			  const struct pingpong_dest *dest)
 {
 	int i;
@@ -96,7 +96,7 @@ static int pp_connect_ctx(struct pingpong_context *ctx, int port, enum ibv_mtu m
 			.ah_attr		= {
 				.is_global	= 0,
 				.dlid		= dest[i].lid,
-				.sl		= 0,
+				.sl		= sl,
 				.src_path_bits	= 0,
 				.port_num	= port
 			}
@@ -157,6 +157,7 @@ static struct pingpong_dest *pp_client_exch_dest(const char *servername, int por
 
 	if (n < 0) {
 		fprintf(stderr, "%s for %s:%d\n", gai_strerror(n), servername, port);
+		free(service);
 		return NULL;
 	}
 
@@ -171,6 +172,7 @@ static struct pingpong_dest *pp_client_exch_dest(const char *servername, int por
 	}
 
 	freeaddrinfo(res);
+	free(service);
 
 	if (sockfd < 0) {
 		fprintf(stderr, "Couldn't connect to %s:%d\n", servername, port);
@@ -214,7 +216,8 @@ out:
 }
 
 static struct pingpong_dest *pp_server_exch_dest(struct pingpong_context *ctx,
-						 int ib_port, enum ibv_mtu mtu, int port,
+						 int ib_port, enum ibv_mtu mtu,
+						 int port, int sl,
 						 const struct pingpong_dest *my_dest)
 {
 	struct addrinfo *res, *t;
@@ -238,6 +241,7 @@ static struct pingpong_dest *pp_server_exch_dest(struct pingpong_context *ctx,
 
 	if (n < 0) {
 		fprintf(stderr, "%s for port %d\n", gai_strerror(n), port);
+		free(service);
 		return NULL;
 	}
 
@@ -256,6 +260,7 @@ static struct pingpong_dest *pp_server_exch_dest(struct pingpong_context *ctx,
 	}
 
 	freeaddrinfo(res);
+	free(service);
 
 	if (sockfd < 0) {
 		fprintf(stderr, "Couldn't listen to port %d\n", port);
@@ -291,7 +296,7 @@ static struct pingpong_dest *pp_server_exch_dest(struct pingpong_context *ctx,
 		       &rem_dest[i].lid, &rem_dest[i].qpn, &rem_dest[i].psn);
 	}
 
-	if (pp_connect_ctx(ctx, ib_port, mtu, my_dest, rem_dest)) {
+	if (pp_connect_ctx(ctx, ib_port, mtu, sl, my_dest, rem_dest)) {
 		fprintf(stderr, "Couldn't connect to remote QP\n");
 		free(rem_dest);
 		rem_dest = NULL;
@@ -408,12 +413,12 @@ static struct pingpong_context *pp_init_ctx(struct ibv_device *ib_dev, int size,
 	}
 
 	for (i = 0; i < num_qp; ++i) {
-		struct ibv_qp_attr attr;
-
-		attr.qp_state        = IBV_QPS_INIT;
-		attr.pkey_index      = 0;
-		attr.port_num        = port;
-		attr.qp_access_flags = 0;
+		struct ibv_qp_attr attr = {
+			.qp_state        = IBV_QPS_INIT,
+			.pkey_index      = 0,
+			.port_num        = port,
+			.qp_access_flags = 0
+		};
 
 		if (ibv_modify_qp(ctx->qp[i], &attr,
 				  IBV_QP_STATE              |
@@ -544,6 +549,7 @@ static void usage(const char *argv0)
 	printf("  -q, --num-qp=<num>     number of QPs to use (default 16)\n");
 	printf("  -r, --rx-depth=<dep>   number of receives to post at a time (default 500)\n");
 	printf("  -n, --iters=<iters>    number of exchanges per QP(default 1000)\n");
+	printf("  -l, --sl=<sl>          service level value\n");
 	printf("  -e, --events           sleep on CQ events (default poll)\n");
 }
 
@@ -571,6 +577,7 @@ int main(int argc, char *argv[])
 	int			 num_wc;
 	int                      i;
 	int                      num_cq_events = 0;
+	int                      sl = 0;
 
 	srand48(getpid() * time(NULL));
 
@@ -586,11 +593,12 @@ int main(int argc, char *argv[])
 			{ .name = "num-qp",   .has_arg = 1, .val = 'q' },
 			{ .name = "rx-depth", .has_arg = 1, .val = 'r' },
 			{ .name = "iters",    .has_arg = 1, .val = 'n' },
+			{ .name = "sl",       .has_arg = 1, .val = 'l' },
 			{ .name = "events",   .has_arg = 0, .val = 'e' },
 			{ 0 }
 		};
 
-		c = getopt_long(argc, argv, "p:d:i:s:m:q:r:n:e", long_options, NULL);
+		c = getopt_long(argc, argv, "p:d:i:s:m:q:r:n:l:e", long_options, NULL);
 		if (c == -1)
 			break;
 
@@ -639,6 +647,10 @@ int main(int argc, char *argv[])
 			iters = strtol(optarg, NULL, 0);
 			break;
 
+		case 'l':
+			sl = strtol(optarg, NULL, 0);
+			break;
+
 		case 'e':
 			++use_event;
 			break;
@@ -726,7 +738,7 @@ int main(int argc, char *argv[])
 	if (servername)
 		rem_dest = pp_client_exch_dest(servername, port, my_dest);
 	else
-		rem_dest = pp_server_exch_dest(ctx, ib_port, mtu, port, my_dest);
+		rem_dest = pp_server_exch_dest(ctx, ib_port, mtu, port, sl, my_dest);
 
 	if (!rem_dest)
 		return 1;
@@ -736,7 +748,7 @@ int main(int argc, char *argv[])
 		       rem_dest[i].lid, rem_dest[i].qpn, rem_dest[i].psn);
 
 	if (servername)
-		if (pp_connect_ctx(ctx, ib_port, mtu, my_dest, rem_dest))
+		if (pp_connect_ctx(ctx, ib_port, mtu, sl, my_dest, rem_dest))
 			return 1;
 
 	if (servername)
@@ -793,7 +805,8 @@ int main(int argc, char *argv[])
 
 			for (i = 0; i < ne; ++i) {
 				if (wc[i].status != IBV_WC_SUCCESS) {
-					fprintf(stderr, "Failed status %d for wr_id %d\n",
+					fprintf(stderr, "Failed status %s (%d) for wr_id %d\n",
+						ibv_wc_status_str(wc[i].status),
 						wc[i].status, (int) wc[i].wr_id);
 					return 1;
 				}
