@@ -248,7 +248,7 @@ int ibv_cmd_reg_mr(struct ibv_pd *pd, void *addr, size_t length,
 	if (write(pd->context->cmd_fd, cmd, cmd_size) != cmd_size)
 		return errno;
 
-	VALGRIND_MAKE_MEM_DEFINED(&resp, sizeof resp);
+	VALGRIND_MAKE_MEM_DEFINED(resp, resp_size);
 
 	mr->handle  = resp->mr_handle;
 	mr->lkey    = resp->lkey;
@@ -291,7 +291,7 @@ static int ibv_cmd_create_cq_v2(struct ibv_context *context, int cqe,
 	if (write(context->cmd_fd, cmd, cmd_size) != cmd_size)
 		return errno;
 
-	VALGRIND_MAKE_MEM_DEFINED(resp, sizeof resp_size);
+	VALGRIND_MAKE_MEM_DEFINED(resp, resp_size);
 
 	cq->handle  = resp->cq_handle;
 	cq->cqe     = resp->cqe;
@@ -432,6 +432,7 @@ int ibv_cmd_destroy_cq(struct ibv_cq *cq)
 
 	IBV_INIT_CMD_RESP(&cmd, sizeof cmd, DESTROY_CQ, &resp, sizeof resp);
 	cmd.cq_handle = cq->handle;
+	cmd.reserved  = 0;
 
 	if (write(cq->context->cmd_fd, &cmd, sizeof cmd) != sizeof cmd)
 		return errno;
@@ -539,10 +540,13 @@ int ibv_cmd_query_srq(struct ibv_srq *srq, struct ibv_srq_attr *srq_attr,
 
 	IBV_INIT_CMD_RESP(cmd, cmd_size, QUERY_SRQ, &resp, sizeof resp);
 	cmd->srq_handle = srq->handle;
+	cmd->reserved   = 0;
 
 	if (write(srq->context->cmd_fd, cmd, cmd_size) != cmd_size)
 		return errno;
 
+	VALGRIND_MAKE_MEM_DEFINED(&resp, sizeof resp);
+
 	srq_attr->max_wr    = resp.max_wr;
 	srq_attr->max_sge   = resp.max_sge;
 	srq_attr->srq_limit = resp.srq_limit;
@@ -573,10 +577,13 @@ int ibv_cmd_destroy_srq(struct ibv_srq *srq)
 
 	IBV_INIT_CMD_RESP(&cmd, sizeof cmd, DESTROY_SRQ, &resp, sizeof resp);
 	cmd.srq_handle = srq->handle;
+	cmd.reserved   = 0;
 
 	if (write(srq->context->cmd_fd, &cmd, sizeof cmd) != sizeof cmd)
 		return errno;
 
+	VALGRIND_MAKE_MEM_DEFINED(&resp, sizeof resp);
+
 	pthread_mutex_lock(&srq->mutex);
 	while (srq->events_completed != resp.events_reported)
 		pthread_cond_wait(&srq->cond, &srq->mutex);
@@ -657,6 +664,8 @@ int ibv_cmd_query_qp(struct ibv_qp *qp, struct ibv_qp_attr *attr,
 	if (write(qp->context->cmd_fd, cmd, cmd_size) != cmd_size)
 		return errno;
 
+	VALGRIND_MAKE_MEM_DEFINED(&resp, sizeof resp);
+
 	attr->qkey                          = resp.qkey;
 	attr->rq_psn                        = resp.rq_psn;
 	attr->sq_psn                        = resp.sq_psn;
@@ -756,6 +765,7 @@ int ibv_cmd_modify_qp(struct ibv_qp *qp, struct ibv_qp_attr *attr,
 	memcpy(cmd->dest.dgid, attr->ah_attr.grh.dgid.raw, 16);
 	cmd->dest.flow_label 	    = attr->ah_attr.grh.flow_label;
 	cmd->dest.dlid 		    = attr->ah_attr.dlid;
+	cmd->dest.reserved	    = 0;
 	cmd->dest.sgid_index 	    = attr->ah_attr.grh.sgid_index;
 	cmd->dest.hop_limit 	    = attr->ah_attr.grh.hop_limit;
 	cmd->dest.traffic_class     = attr->ah_attr.grh.traffic_class;
@@ -768,6 +778,7 @@ int ibv_cmd_modify_qp(struct ibv_qp *qp, struct ibv_qp_attr *attr,
 	memcpy(cmd->alt_dest.dgid, attr->alt_ah_attr.grh.dgid.raw, 16);
 	cmd->alt_dest.flow_label    = attr->alt_ah_attr.grh.flow_label;
 	cmd->alt_dest.dlid 	    = attr->alt_ah_attr.dlid;
+	cmd->alt_dest.reserved	    = 0;
 	cmd->alt_dest.sgid_index    = attr->alt_ah_attr.grh.sgid_index;
 	cmd->alt_dest.hop_limit     = attr->alt_ah_attr.grh.hop_limit;
 	cmd->alt_dest.traffic_class = attr->alt_ah_attr.grh.traffic_class;
@@ -882,7 +893,8 @@ int ibv_cmd_post_send(struct ibv_qp *ibqp, struct ibv_send_wr *wr,
 		while (--wr_count)
 			i = i->next;
 		*bad_wr = i;
-	}
+	} else if (ret)
+		*bad_wr = wr;
 
 	return ret;
 }
@@ -942,7 +954,8 @@ int ibv_cmd_post_recv(struct ibv_qp *ibqp, struct ibv_recv_wr *wr,
 		while (--wr_count)
 			i = i->next;
 		*bad_wr = i;
-	}
+	} else if (ret)
+		*bad_wr = wr;
 
 	return ret;
 }
@@ -1002,7 +1015,8 @@ int ibv_cmd_post_srq_recv(struct ibv_srq *srq, struct ibv_recv_wr *wr,
 		while (--wr_count)
 			i = i->next;
 		*bad_wr = i;
-	}
+	} else if (ret)
+		*bad_wr = wr;
 
 	return ret;
 }
@@ -1062,6 +1076,7 @@ int ibv_cmd_destroy_qp(struct ibv_qp *qp)
 
 	IBV_INIT_CMD_RESP(&cmd, sizeof cmd, DESTROY_QP, &resp, sizeof resp);
 	cmd.qp_handle = qp->handle;
+	cmd.reserved  = 0;
 
 	if (write(qp->context->cmd_fd, &cmd, sizeof cmd) != sizeof cmd)
 		return errno;
@@ -1084,6 +1099,7 @@ int ibv_cmd_attach_mcast(struct ibv_qp *qp, union ibv_gid *gid, uint16_t lid)
 	memcpy(cmd.gid, gid->raw, sizeof cmd.gid);
 	cmd.qp_handle = qp->handle;
 	cmd.mlid      = lid;
+	cmd.reserved  = 0;
 
 	if (write(qp->context->cmd_fd, &cmd, sizeof cmd) != sizeof cmd)
 		return errno;
@@ -1099,6 +1115,7 @@ int ibv_cmd_detach_mcast(struct ibv_qp *qp, union ibv_gid *gid, uint16_t lid)
 	memcpy(cmd.gid, gid->raw, sizeof cmd.gid);
 	cmd.qp_handle = qp->handle;
 	cmd.mlid      = lid;
+	cmd.reserved  = 0;
 
 	if (write(qp->context->cmd_fd, &cmd, sizeof cmd) != sizeof cmd)
 		return errno;
