@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2004, 2005 Topspin Communications.  All rights reserved.
+ * Copyright (c) 2006, 2007 Cisco Systems, Inc.  All rights reserved.
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -28,8 +29,6 @@
  * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
- *
- * $Id: device.c 7621 2006-06-01 23:16:18Z roland $
  */
 
 #if HAVE_CONFIG_H
@@ -53,7 +52,7 @@ static pthread_mutex_t device_list_lock = PTHREAD_MUTEX_INITIALIZER;
 static int num_devices;
 static struct ibv_device **device_list;
 
-struct ibv_device **ibv_get_device_list(int *num)
+struct ibv_device **__ibv_get_device_list(int *num)
 {
 	struct ibv_device **l;
 	int i;
@@ -64,9 +63,13 @@ struct ibv_device **ibv_get_device_list(int *num)
 		num_devices = ibverbs_init(&device_list);
 
 	l = calloc(num_devices + 1, sizeof (struct ibv_device *));
+	if (!l)
+		goto out;
+
 	for (i = 0; i < num_devices; ++i)
 		l[i] = device_list[i];
 
+out:
 	pthread_mutex_unlock(&device_list_lock);
 
 	if (num)
@@ -74,25 +77,28 @@ struct ibv_device **ibv_get_device_list(int *num)
 
 	return l;
 }
+default_symver(__ibv_get_device_list, ibv_get_device_list);
 
-void ibv_free_device_list(struct ibv_device **list)
+void __ibv_free_device_list(struct ibv_device **list)
 {
 	free(list);
 }
+default_symver(__ibv_free_device_list, ibv_free_device_list);
 
-const char *ibv_get_device_name(struct ibv_device *device)
+const char *__ibv_get_device_name(struct ibv_device *device)
 {
-	return device->ibdev->name;
+	return device->name;
 }
+default_symver(__ibv_get_device_name, ibv_get_device_name);
 
-uint64_t ibv_get_device_guid(struct ibv_device *device)
+uint64_t __ibv_get_device_guid(struct ibv_device *device)
 {
 	char attr[24];
 	uint64_t guid = 0;
 	uint16_t parts[4];
 	int i;
 
-	if (ibv_read_sysfs_file(device->ibdev->path, "node_guid",
+	if (ibv_read_sysfs_file(device->ibdev_path, "node_guid",
 				attr, sizeof attr) < 0)
 		return 0;
 
@@ -105,14 +111,16 @@ uint64_t ibv_get_device_guid(struct ibv_device *device)
 
 	return htonll(guid);
 }
+default_symver(__ibv_get_device_guid, ibv_get_device_guid);
 
-struct ibv_context *ibv_open_device(struct ibv_device *device)
+struct ibv_context *__ibv_open_device(struct ibv_device *device)
 {
 	char *devpath;
 	int cmd_fd;
 	struct ibv_context *context;
 
-	asprintf(&devpath, "/dev/infiniband/%s", device->dev->name);
+	if (asprintf(&devpath, "/dev/infiniband/%s", device->dev_name) < 0)
+		return NULL;
 
 	/*
 	 * We'll only be doing writes, but we need O_RDWR in case the
@@ -130,6 +138,7 @@ struct ibv_context *ibv_open_device(struct ibv_device *device)
 
 	context->device = device;
 	context->cmd_fd = cmd_fd;
+	pthread_mutex_init(&context->mutex, NULL);
 
 	return context;
 
@@ -138,8 +147,9 @@ err:
 
 	return NULL;
 }
+default_symver(__ibv_open_device, ibv_open_device);
 
-int ibv_close_device(struct ibv_context *context)
+int __ibv_close_device(struct ibv_context *context)
 {
 	int async_fd = context->async_fd;
 	int cmd_fd   = context->cmd_fd;
@@ -160,9 +170,10 @@ int ibv_close_device(struct ibv_context *context)
 
 	return 0;
 }
+default_symver(__ibv_close_device, ibv_close_device);
 
-int ibv_get_async_event(struct ibv_context *context,
-			struct ibv_async_event *event)
+int __ibv_get_async_event(struct ibv_context *context,
+			  struct ibv_async_event *event)
 {
 	struct ibv_kern_async_event ev;
 
@@ -197,10 +208,14 @@ int ibv_get_async_event(struct ibv_context *context,
 		break;
 	}
 
+	if (context->ops.async_event)
+		context->ops.async_event(event);
+
 	return 0;
 }
+default_symver(__ibv_get_async_event, ibv_get_async_event);
 
-void ibv_ack_async_event(struct ibv_async_event *event)
+void __ibv_ack_async_event(struct ibv_async_event *event)
 {
 	switch (event->event_type) {
 	case IBV_EVENT_CQ_ERR:
@@ -251,3 +266,4 @@ void ibv_ack_async_event(struct ibv_async_event *event)
 		return;
 	}
 }
+default_symver(__ibv_ack_async_event, ibv_ack_async_event);
